{% load crispy_forms_tags %}
{% load static %}
<style type="text/css">
    .dynamic-form > .row > label > .asteriskField {
    display: none;
}
</style>
{{ formset.management_form }}

    {% for form in formset.forms %}
    <div class="{% cycle 'row1' 'row2' %} formset_row-{{ formset.prefix }}">
            <tr>
                {% for field in form.visible_fields %}
                <td>
                    {# Include the hidden fields in the form #}
                    {% if forloop.first %}
                        {% for hidden in form.hidden_fields %}
                            {{ hidden }}
                        {% endfor %}
                    {% endif %}
                   <!--  {{ field.errors.as_ul }} -->
                    {{ field|as_crispy_field }}
                </td>
                {% endfor %}
            </tr>
    </div>
    {% endfor %}
<script src="{% static 'webpage/libraries/django-dynamic-formset/jquery.formset.js' %}"></script>
<script type="text/javascript">
    $('.formset_row-{{ formset.prefix }}').formset({
        addText: 'add another',
        deleteText: 'remove',
        prefix: '{{ formset.prefix }}',
    });
</script>
